﻿package com.fynas.bookplan;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.fynas.android.util.DateTime;
import com.fynas.android.util.FLog;
import com.fynas.android.util.HttpRequest;
import com.fynas.bookplan.common.BaseActivity;
import com.fynas.bookplan.common.Config;
import com.fynas.android.model.JsonResult;
import com.fynas.bookplan.model.Plan;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;

import android.os.Bundle;
import android.app.AlertDialog;
import android.app.DatePickerDialog;
import android.app.AlertDialog.Builder;
import android.app.DatePickerDialog.OnDateSetListener;
import android.content.DialogInterface;
import android.content.Intent;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.ProgressBar;
import android.widget.Toast;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.DatePicker;
import android.widget.EditText;

public class AddActivity extends BaseActivity {

	private Button mAddPlanButton;
	private EditText mStartDateText;
	private EditText mPlanEndDateText;
	private AutoCompleteTextView mBookNameAutoText;
	private CheckBox mIsWantCheck;

	private String mStartDateCache;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.activity_add);
		// 初始化成员变量
		mAddPlanButton = (Button) findViewById(R.id.add_plan);
		mStartDateText = (EditText) findViewById(R.id.add_date_start);
		mPlanEndDateText = (EditText) findViewById(R.id.add_date_plan_end);
		// 注册suggestion事件
		mBookNameAutoText = (AutoCompleteTextView) this
				.findViewById(R.id.new_book_name);
		mBookNameAutoText.addTextChangedListener(suggestionWatcher);
		mBookNameAutoText
				.setOnItemSelectedListener(suggestItemSelectedListener);

		// 切换在读和想读状态
		mIsWantCheck = (CheckBox) this.findViewById(R.id.is_want);
		mIsWantCheck.setOnCheckedChangeListener(isWantCheckedChangeListener);
		// 初始化默认读完时间
		Calendar calendar = Calendar.getInstance();
		mStartDateCache = DateTime.calendarToString(calendar);
		mStartDateText.setText(mStartDateCache);
		// 初始化计划读完时间，默认1个月
		calendar.add(Calendar.MONTH, 1);
		mPlanEndDateText.setText(DateTime.calendarToString(calendar));

		// 挂载弹出日期选择框事件
		mStartDateText.setOnClickListener(startDateClickListener);
		mPlanEndDateText.setOnClickListener(planEndDateClickListener);

		// 注册添加plan事件
		mAddPlanButton.setOnClickListener(addClickListener);
	}

	/**
	 * 创建菜单
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		getMenuInflater().inflate(R.menu.activity_add, menu);
		return true;
	}

	/**
	 * 点击菜单
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		if (item.getItemId() == R.id.menu_cancel) {
			AlertDialog.Builder builder = new Builder(this);
			builder.setTitle(getResources().getString(R.string.notice));
			builder.setMessage(getResources().getString(
					R.string.add_confirm_cancel));
			builder.setPositiveButton(getResources().getString(R.string.ok),
					cancelOkClickListener);

			builder.setNegativeButton(
					getResources().getString(R.string.cancel), null);
			builder.create().show();
		}
		return true;

	}

	/**
	 * 确定取消添加计划
	 */
	private DialogInterface.OnClickListener cancelOkClickListener = new DialogInterface.OnClickListener() {

		@Override
		public void onClick(DialogInterface dialog, int which) {
			finish();
		}
	};

	/**
	 * suggestion中的一项被选中
	 */
	private OnItemSelectedListener suggestItemSelectedListener = new OnItemSelectedListener() {

		@Override
		public void onItemSelected(AdapterView<?> adapterView, View view,
				int position, long id) {

		}

		@Override
		public void onNothingSelected(AdapterView<?> adapterView) {

		}

	};
	/**
	 * 切换是否为想读
	 */
	private OnCheckedChangeListener isWantCheckedChangeListener = new OnCheckedChangeListener() {

		@Override
		public void onCheckedChanged(CompoundButton button, boolean checked) {
			mAddPlanButton.setText(getResources().getString(
					checked ? R.string.add_add_to_want
							: R.string.add_add_to_reading));
			if (checked) {
				mStartDateText.setText("暂不开始");
				mStartDateText.setEnabled(false);
			} else {
				mStartDateText.setText(mStartDateCache);
				mStartDateText.setEnabled(true);
			}
		}

	};

	private OnDateSetListener startDateSetListener = new OnDateSetListener() {

		@Override
		public void onDateSet(DatePicker view, int year, int monthOfYear,
				int dayOfMonth) {
			mStartDateCache = DateTime.toString(year, monthOfYear, dayOfMonth);
			mStartDateText.setText(mStartDateCache);
		}
	};

	private OnDateSetListener planEndDateSetListener = new OnDateSetListener() {

		@Override
		public void onDateSet(DatePicker view, int year, int monthOfYear,
				int dayOfMonth) {
			mPlanEndDateText.setText(DateTime.toString(year, monthOfYear,
					dayOfMonth));
		}

	};

	/**
	 * 点击开始读日期
	 */
	private OnClickListener startDateClickListener = new OnClickListener() {

		@Override
		public void onClick(View view) {
			Calendar calendar = DateTime.stringToCalendar(mStartDateText
					.getText().toString());
			if (calendar.get(Calendar.YEAR) < 2000) {
				calendar = Calendar.getInstance();
			}
			new DatePickerDialog(AddActivity.this, startDateSetListener,
					calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH),
					calendar.get(Calendar.DAY_OF_MONTH)).show();
		}

	};

	/**
	 * 点击计划结束日期
	 */
	private OnClickListener planEndDateClickListener = new OnClickListener() {

		@Override
		public void onClick(View view) {
			Calendar calendar = DateTime.stringToCalendar(mPlanEndDateText
					.getText().toString());
			if (calendar.get(Calendar.YEAR) < 2000) {
				calendar = Calendar.getInstance();
			}
			new DatePickerDialog(AddActivity.this, planEndDateSetListener,
					calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH),
					calendar.get(Calendar.DAY_OF_MONTH)).show();
		}

	};

	/**
	 * 点击添加按钮
	 */
	private OnClickListener addClickListener = new OnClickListener() {

		@Override
		public void onClick(View view) {
			// 显示正在加载图标
			ProgressBar loading = (ProgressBar) findViewById(R.id.add_plan_progress);
			loading.setVisibility(View.VISIBLE);

			// 后台添加数据
			RequestParams params = new RequestParams();
			params.put("name", mBookNameAutoText.getText().toString());
			params.put("plan_end_date", mPlanEndDateText.getText().toString());
			if (mIsWantCheck.isChecked()) {
				params.put("status", Integer.toString(Plan.STATUS_WANT));
			} else {
				params.put("status", Integer.toString(Plan.STATUS_READING));
				params.put("start_date", mStartDateText.getText().toString());
			}

			HttpRequest.post(Config.Api.URL_ADD_PLAN, params,
					new AsyncHttpResponseHandler() {
						public void onSuccess(String response) {
							JsonResult result = new JsonResult(response);
							if (result.isOk()) {
								Toast.makeText(AddActivity.this, "添加成功",
										Toast.LENGTH_SHORT).show();
								startActivity(new Intent(AddActivity.this,
										MainActivity.class));
								finish();
							} else {
								Toast.makeText(AddActivity.this, "添加失败",
										Toast.LENGTH_SHORT).show();
								FLog.e(response);
							}

						}

						public void onFinish() {
							// 隐藏loading图标
							ProgressBar loading = (ProgressBar) findViewById(R.id.add_plan_progress);
							loading.setVisibility(View.GONE);
						}
					});
		}

	};

	private TextWatcher suggestionWatcher = new TextWatcher() {

		@Override
		public void afterTextChanged(Editable arg0) {
			// TODO Auto-generated method stub

		}

		@Override
		public void beforeTextChanged(CharSequence arg0, int arg1, int arg2,
				int arg3) {
			// TODO Auto-generated method stub

		}

		@Override
		public void onTextChanged(CharSequence s, int start, int before,
				int count) {
			if (mBookNameAutoText.enoughToFilter()) {
				String keyword;
				try {
					keyword = URLEncoder.encode(s.toString().trim(), "UTF-8");
					HttpRequest.get(Config.Api.URL_BOOK_SUGGESTION,
							new RequestParams("k", keyword),
							new AsyncHttpResponseHandler() {
								public void onSuccess(String response) {
									List<String> suggestion = new ArrayList<String>();
									JsonResult result = new JsonResult(response);
									AutoCompleteTextView bookNameAutoText = (AutoCompleteTextView) findViewById(R.id.new_book_name);
									try {
										JSONArray books = result
												.getJsonObject().getJSONArray(
														"data");
										for (int i = 0; i < books.length(); i++) {
											JSONObject book = (JSONObject) books
													.get(i);
											suggestion.add(book
													.getString("name"));
										}

									} catch (JSONException e) {
										e.printStackTrace();
									}
									ArrayAdapter<String> suggestionAdapter = new ArrayAdapter<String>(
											AddActivity.this,
											R.layout.suggestion_item,
											suggestion);
									bookNameAutoText
											.setAdapter(suggestionAdapter);
								}
							});
				} catch (UnsupportedEncodingException e) {
					e.printStackTrace();
				}
			}
			// TODO 优化：当上一次就没有命中suggestion时，就不再获取suggestion了
			// TODO 优化：当用户从suggestion列表中选中一项时不发suggestion请求
		}
	};

}
