﻿package com.fynas.bookplan;

import java.util.HashMap;
import java.util.Map;

import org.json.JSONException;
import org.json.JSONObject;

import com.fynas.android.util.FLog;
import com.fynas.android.util.HttpRequest;
import com.fynas.android.util.PackageUtil;
import com.fynas.bookplan.common.BaseActivity;
import com.fynas.bookplan.common.Config;
import com.fynas.bookplan.common.User;
import com.fynas.android.model.JsonResult;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;

import android.net.Uri;
import android.os.Bundle;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.DialogInterface;
import android.content.Intent;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ProgressBar;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ListView;

public class HelpActivity extends BaseActivity {

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_help);

		// 初始化list view
		ListView functionListView = (ListView) findViewById(R.id.more_function_list);
		MoreFunctionAdapter adapter = new MoreFunctionAdapter(this);
		functionListView.setAdapter(adapter);
		functionListView.setOnItemClickListener(functionItemClickListener);

	}

	/**
	 * 点击更多功能中的一项
	 */
	OnItemClickListener functionItemClickListener = new OnItemClickListener() {

		@Override
		public void onItemClick(AdapterView<?> adapterView, View view,
				int position, long id) {
			if (id == MoreFunctionAdapter.FUNCTION_ID_CHECK_UPDATE) {
				checkUpdate();
			} else if (id == MoreFunctionAdapter.FUNCTION_ID_ABOUT) {
				about();
			} else if (id == MoreFunctionAdapter.FUNCTION_ID_WEIBO) {

			} else if (id == MoreFunctionAdapter.FUNCTION_ID_FEED_BACK) {
				startActivity(new Intent(HelpActivity.this,
						FeedbackActivity.class));
			} else if (id == MoreFunctionAdapter.FUNCTION_ID_LOGOUT) {
				logout();
			}

		}

	};

	/**
	 * 检查版本更新
	 */
	private void checkUpdate() {
		final ProgressBar loading = (ProgressBar) findViewById(R.id.help_progress);
		loading.setVisibility(View.VISIBLE);
		RequestParams params = new RequestParams();
		params.put("v", PackageUtil.getVersionName(getApplicationContext()));
		HttpRequest.get(Config.Api.URL_CHECK_UPDATE, params,
				new AsyncHttpResponseHandler() {
					@Override
					public void onFinish() {
						loading.setVisibility(View.GONE);
					}

					@Override
					public void onFailure(Throwable error, String content) {
						Toast.makeText(
								HelpActivity.this,
								getResources().getString(
										R.string.help_no_need_update),
								Toast.LENGTH_SHORT).show();
					}

					@Override
					public void onSuccess(String response) {
						FLog.d(response);
						JsonResult result = new JsonResult(response);
						if (result.isOk() && result.getJsonObject().has("data")) {
							try {
								JSONObject detail = result.getJsonObject()
										.getJSONObject("data");
								Map<String, String> appInfo = new HashMap<String, String>();
								appInfo.put("version",
										detail.getString("version"));
								appInfo.put("description",
										detail.getString("description"));
								appInfo.put("date", detail.getString("date"));

								AlertDialog.Builder builder = new Builder(
										HelpActivity.this);
								builder.setTitle(HelpActivity.this
										.getResources()
										.getString(
												R.string.help_new_version_notice));

								builder.setMessage("检测到新版本v"
										+ appInfo.get("version") + "\n"
										+ appInfo.get("description"));

								builder.setPositiveButton(
										HelpActivity.this
												.getResources()
												.getString(
														R.string.help_update_now),
										updateClickListener);

								builder.setNegativeButton(
										HelpActivity.this
												.getResources()
												.getString(
														R.string.help_update_later),
										null);
								builder.create().show();

							} catch (JSONException e) {
								Toast.makeText(
										HelpActivity.this,
										getResources().getString(
												R.string.help_no_need_update),
										Toast.LENGTH_SHORT).show();
								e.printStackTrace();
							}

						}
					}
				});
	}

	/**
	 * 点击升级确认对话框中的确定
	 */
	private DialogInterface.OnClickListener updateClickListener = new DialogInterface.OnClickListener() {

		@Override
		public void onClick(DialogInterface dialog, int which) {
			startActivity(new Intent("android.intent.action.VIEW",
					Uri.parse(Config.Api.URL_DOWNLOAD_APP)));
		}

	};

	/**
	 * 关于
	 */
	private void about() {
		AlertDialog.Builder builder = new Builder(this);
		builder.setTitle(getResources().getString(R.string.help_about));
		builder.setMessage(getResources().getString(R.string.help_version)
				+ PackageUtil.getVersionName(getApplicationContext()));
		builder.setPositiveButton(getResources().getString(R.string.ok), null);
		builder.create().show();
	}

	/**
	 * 注销登录
	 */
	private void logout() {
		AlertDialog.Builder builder = new Builder(this);
		builder.setMessage(getResources().getString(R.string.confirm_logout));
		builder.setTitle(getResources().getString(R.string.notice));
		builder.setPositiveButton(getResources().getString(R.string.ok),
				logoutOkClickListener);

		builder.setNegativeButton(getResources().getString(R.string.cancel),
				null);
		builder.create().show();
	}

	/**
	 * 确定退出登录
	 */
	private DialogInterface.OnClickListener logoutOkClickListener = new DialogInterface.OnClickListener() {

		@Override
		public void onClick(DialogInterface dialog, int which) {
			// 清空cookie
			User user = User.getInstance(HelpActivity.this);
			user.logout();
			// 返回登录界面
			Intent intent = new Intent(HelpActivity.this, LoginActivity.class);
			intent.putExtra("start_activity_after_login",
					MainActivity.class.getName());
			startActivity(intent);
			finish();
		}

	};

}
