﻿package com.fynas.bookplan;

import java.util.ArrayList;
import java.util.List;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.baidu.mobstat.StatService;
import com.fynas.android.util.DateTime;
import com.fynas.android.util.FLog;
import com.fynas.android.util.HttpRequest;
import com.fynas.bookplan.R;
import com.fynas.bookplan.common.ApplicationHelper;
import com.fynas.bookplan.common.Config;
import com.fynas.android.model.JsonResult;
import com.fynas.bookplan.model.Plan;
import com.fynas.bookplan.model.PlanReminder;
import com.fynas.bookplan.view.PullToRefreshListView;
import com.fynas.bookplan.view.PullToRefreshListView.OnRefreshListener;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;

import android.os.Bundle;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.AlertDialog.Builder;
import android.app.TabActivity;
import android.content.DialogInterface;
import android.content.Intent;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.View.OnClickListener;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView;
import android.widget.HeaderViewListAdapter;
import android.widget.ProgressBar;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.TabHost.OnTabChangeListener;

public class MainActivity extends TabActivity {

	public static final String INTENT_EXTRA_ERROR_MESSAGE = "error_message";

	private TabHost mTabHost;
	private ProgressBar mLoading;
	private PullToRefreshListView mReadingListView;
	private PullToRefreshListView mWantListView;
	private PullToRefreshListView mReadListView;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// 初始化TabHost
		mTabHost = this.getTabHost();
		LayoutInflater.from(this).inflate(R.layout.activity_main,
				mTabHost.getTabContentView(), true);
		mTabHost.addTab(mTabHost.newTabSpec(Config.TAB_TAG_READING)
				.setIndicator(getResources().getString(R.string.reading), null)
				.setContent(R.id.reading_list));
		mTabHost.addTab(mTabHost
				.newTabSpec(Config.TAB_TAG_WANT)
				.setIndicator(getResources().getString(R.string.want_read),
						null).setContent(R.id.wanted_list));
		mTabHost.addTab(mTabHost
				.newTabSpec(Config.TAB_TAG_READ)
				.setIndicator(getResources().getString(R.string.has_read), null)
				.setContent(R.id.read_list));

		mTabHost.setOnTabChangedListener(tabChangeListener);

		// 获取ProgressBar
		mLoading = (ProgressBar) findViewById(R.id.loading_progress);
		// 加载在读数据
		loadTabData(Config.TAB_TAG_READING, false, false);

		// 挂载下拉刷新事件
		mReadingListView = (PullToRefreshListView) findViewById(R.id.reading_list);
		mReadingListView.setOnRefreshListener(readingRefreshListener);

		mWantListView = (PullToRefreshListView) findViewById(R.id.wanted_list);
		mWantListView.setOnRefreshListener(readingRefreshListener);

		mReadListView = (PullToRefreshListView) findViewById(R.id.read_list);
		mReadListView.setOnRefreshListener(readingRefreshListener);

		// 获取读书提醒
		HttpRequest.get(Config.Api.URL_GET_PLAN_REMIND, new RequestParams(),
				new AsyncHttpResponseHandler() {
					public void onSuccess(String response) {
						JsonResult result = new JsonResult(response);
						if (result.isOk()) {
							// TODO:remind
						}
					}
				});

		ApplicationHelper.getInstance().addActivity(this);
	}

	/**
	 * 提醒
	 * 
	 * @param reminders
	 */
	private void remindPlan(List<PlanReminder> reminders) {
		NotificationManager manager = (NotificationManager) getSystemService(Activity.NOTIFICATION_SERVICE);
		Notification notification = new Notification();
		notification.defaults = Notification.DEFAULT_SOUND;
		notification.icon = R.drawable.ic_launcher;
		notification.when = System.currentTimeMillis();
		notification.flags |= Notification.FLAG_AUTO_CANCEL;// 设置点击后自动清除通知

		PendingIntent contentIntent = PendingIntent.getActivity(this, 0,
				new Intent(this, PlanRemindActivity.class), 0);
		notification.setLatestEventInfo(this,
				getResources().getString(R.string.app_name), "有一本书即将到期",
				contentIntent);
		manager.notify(Config.Notification.PLAN, notification);
	}

	/**
	 * 下拉刷新在读列表
	 */
	OnRefreshListener readingRefreshListener = new OnRefreshListener() {
		public void onRefresh() {
			loadTabData(mTabHost.getCurrentTabTag(), true, false);
		}
	};

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		getMenuInflater().inflate(R.menu.activity_main, menu);

		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		if (item.getItemId() == R.id.menu_settings) {
			startActivity(new Intent(this, SettingsActivity.class));
		} else if (item.getItemId() == R.id.menu_add) {
			startActivity(new Intent(this, AddActivity.class));
			StatService.onEvent(this, Config.Stat.MENU_ADD_PLAN,
					Config.Stat.MENU_ADD_PLAN);
		} else if (item.getItemId() == R.id.menu_achievement) {
			startActivity(new Intent(this, AchievementActivity.class));
		} else if (item.getItemId() == R.id.menu_more) {
			startActivity(new Intent(this, HelpActivity.class));
			StatService.onEvent(this, Config.Stat.MENU_MORE,
					Config.Stat.MENU_MORE);
		} else if (item.getItemId() == R.id.menu_quit) {
			StatService.onEvent(this, Config.Stat.MENU_QUIT,
					Config.Stat.MENU_QUIT);
			ApplicationHelper.getInstance().exit(this);
		}
		return true;

	}

	@Override
	public void onCreateContextMenu(ContextMenu menu, View view,
			ContextMenuInfo menuInfo) {
		menu.setHeaderTitle(R.string.operation);
		getMenuInflater().inflate(R.menu.context_plan_item, menu);

		menu.getItem(0).setVisible(
				mTabHost.getCurrentTabTag() == Config.TAB_TAG_READ ? false
						: true);
	}

	/**
	 * 上下文菜单被点击
	 */
	@Override
	public boolean onContextItemSelected(MenuItem item) {
		if (item.getItemId() == R.id.context_delete_plan) {// 删除条目
			deleteItem((AdapterContextMenuInfo) item.getMenuInfo());
		} else if (item.getItemId() == R.id.context_edit_plan) {// 编辑条目
			Intent intent = new Intent(this, PlanDetailActivity.class);
			intent.putExtra("id",
					((AdapterContextMenuInfo) item.getMenuInfo()).id);
			intent.putExtra("is_auto_enter_edit", true);
			startActivity(intent);
		}
		return true;
	}

	@Override
	protected void onDestroy() {
		ApplicationHelper.getInstance().removeActivity(this);
		super.onDestroy();
	}

	/**
	 * 删除计划条目
	 * 
	 * @param item
	 */
	private void deleteItem(final AdapterContextMenuInfo menuInfo) {
		AlertDialog.Builder builder = new Builder(this);
		builder.setMessage(getResources().getString(R.string.confirm_delete));
		builder.setTitle(getResources().getString(R.string.notice));
		builder.setPositiveButton(getResources().getString(R.string.ok),
				new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						HeaderViewListAdapter headerAdapter = (HeaderViewListAdapter) ((PullToRefreshListView) menuInfo.targetView
								.getParent()).getAdapter();
						PlanListAdapter adapter = (PlanListAdapter) headerAdapter
								.getWrappedAdapter();
						// 从UI界面中删除条目
						adapter.getPlanList().remove(menuInfo.position - 1);
						adapter.notifyDataSetChanged();
						// 从数据库中删除
						HttpRequest.post(Config.Api.URL_DELETE_PLAN,
								new RequestParams("id", menuInfo.id),
								new AsyncHttpResponseHandler() {
									public void onSuccess(String response) {
										JsonResult result = new JsonResult(
												response);
										if (!result.isOk()) {
											Toast.makeText(MainActivity.this,
													"删除失败，请刷新后重试",
													Toast.LENGTH_SHORT).show();
										}
									}
								});
					}

				});

		builder.setNegativeButton(getResources().getString(R.string.cancel),
				null);

		builder.create().show();
	}

	/**
	 * 切换Tab页
	 */
	private OnTabChangeListener tabChangeListener = new OnTabChangeListener() {
		@Override
		public void onTabChanged(String tag) {
			if (tag == Config.TAB_TAG_WANT) {
				((TextView) (MainActivity.this)
						.findViewById(R.id.data_empty_text))
						.setText(R.string.no_want_books);
				MainActivity.this.findViewById(R.id.add_plan_now_button)
						.setVisibility(View.VISIBLE);
			} else if (tag == Config.TAB_TAG_READ) {
				((TextView) (MainActivity.this)
						.findViewById(R.id.data_empty_text))
						.setText(R.string.no_read_books);
				MainActivity.this.findViewById(R.id.add_plan_now_button)
						.setVisibility(View.GONE);
			} else {
				((TextView) (MainActivity.this)
						.findViewById(R.id.data_empty_text))
						.setText(R.string.no_reading_books);
				MainActivity.this.findViewById(R.id.add_plan_now_button)
						.setVisibility(View.VISIBLE);
			}
			loadTabData(tag, false, false);
		}
	};

	/**
	 * 加载标签数据
	 * 
	 * @param tag
	 *            标签Tag
	 * @param refresh
	 *            切换标签时是否需要强制刷新数据
	 * @param cancelOther
	 *            切换标签时是否要停止其它标签加载数据的任务
	 */
	private void loadTabData(final String tag, boolean refresh,
			boolean cancelOther) {
		mLoading.setVisibility(View.VISIBLE);
		findViewById(R.id.data_empty_layout).setVisibility(View.GONE);
		String url;
		if (tag.equalsIgnoreCase(Config.TAB_TAG_READ)) {
			url = Config.Api.URL_GET_READ;
		} else if (tag.equalsIgnoreCase(Config.TAB_TAG_WANT)) {
			url = Config.Api.URL_GET_WANT;
		} else {
			url = Config.Api.URL_GET_READING;
		}

		HttpRequest.get(url, new RequestParams(),
				new AsyncHttpResponseHandler() {

					public void onFinish() {
						mLoading.setVisibility(View.GONE);
					}

					public void onSuccess(String response) {
						List<Plan> plans = new ArrayList<Plan>();
						JsonResult result = new JsonResult(response);
						if (result.isOk()) {
							try {
								JSONArray books = result.getJsonObject()
										.getJSONArray("data");
								for (int i = 0; i < books.length(); i++) {
									Plan plan = new Plan();
									plan.parseJson((JSONObject) books.get(i));
									plans.add(plan);
									fill(tag, plans);
								}
							} catch (JSONException e) {
								e.printStackTrace();
							}
						} else {
							FLog.e(response);
						}
					}

					public void onFailure(Throwable e) {
						Toast.makeText(getApplicationContext(),
								R.string.error_load_data, Toast.LENGTH_SHORT)
								.show();
						FLog.e(e.getMessage());
					}

				});

	}

	private void fill(String tag, List<Plan> plans) {
		// 填充数据
		PullToRefreshListView planListView;
		if (tag == Config.TAB_TAG_WANT) {
			planListView = (PullToRefreshListView) findViewById(R.id.wanted_list);
		} else if (tag == Config.TAB_TAG_READ) {
			planListView = (PullToRefreshListView) findViewById(R.id.read_list);
		} else {
			planListView = (PullToRefreshListView) findViewById(R.id.reading_list);
		}
		PlanListAdapter plansAdapter = new PlanListAdapter(this, plans);
		planListView.setAdapter(plansAdapter);

		// 注册上下文菜单
		registerForContextMenu(planListView);

		// 隐藏loading图标
		ProgressBar loading = (ProgressBar) findViewById(R.id.loading_progress);
		loading.setVisibility(View.GONE);
		// 挂载点击事件
		planListView.setOnItemClickListener(planItemClickListener);

		planListView.onRefreshComplete(DateTime.nowString("MM-dd HH:mm:ss"));
		// 显示或隐藏无数据提示

		findViewById(R.id.data_empty_layout).setVisibility(
				plans == null ? View.VISIBLE : View.GONE);
		findViewById(R.id.add_plan_now_button).setOnClickListener(
				plans == null ? addPlanNowClickListener : null);
	}

	/**
	 * 点击现在添加按钮
	 */
	private OnClickListener addPlanNowClickListener = new OnClickListener() {

		@Override
		public void onClick(View view) {
			startActivity(new Intent(MainActivity.this, AddActivity.class));
		}

	};

	/**
	 * 点击列表的一项
	 */
	private OnItemClickListener planItemClickListener = new OnItemClickListener() {

		@Override
		public void onItemClick(AdapterView<?> adapterView, View view,
				int position, long id) {
			Intent intent = new Intent(MainActivity.this,
					PlanDetailActivity.class);
			intent.putExtra("id", id);
			startActivity(intent);
		}

	};

}
