﻿package com.fynas.bookplan;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.app.Activity;
import android.content.Context;
import android.graphics.drawable.BitmapDrawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

public class MoreFunctionAdapter extends BaseAdapter {

	public static final long FUNCTION_ID_WEIBO = 0;
	public static final long FUNCTION_ID_CHECK_UPDATE = 1;
	public static final long FUNCTION_ID_ABOUT = 2;
	public static final long FUNCTION_ID_FEED_BACK = 3;
	public static final long FUNCTION_ID_LOGOUT = 4;

	private LayoutInflater mLayout; // 视图容器
	private static List<Map<String, Object>> mFunctionList;

	public final class ListItemHolder { // 自定义控件集合
		public ImageView iconImage;
		public TextView titleText;
		public ImageView detailImage;
	}

	public MoreFunctionAdapter(Context context) {
		mLayout = LayoutInflater.from(context);
		// 初始化数据
		if (mFunctionList == null) {
			mFunctionList = new ArrayList<Map<String, Object>>();

			// weibo功能
			Map<String, Object> weiboItem = new HashMap<String, Object>();
			weiboItem.put("icon", ((Activity) context).getResources()
					.getDrawable(R.drawable.weibo_icon));
			weiboItem.put("title", ((Activity) context).getResources()
					.getString(R.string.help_weibo));
			weiboItem.put("show_detail_icon", false);
			mFunctionList.add(weiboItem);

			// 检查更新功能
			Map<String, Object> checkUpdateItem = new HashMap<String, Object>();
			checkUpdateItem.put("icon", ((Activity) context).getResources()
					.getDrawable(R.drawable.update_icon));
			checkUpdateItem.put("title", ((Activity) context).getResources()
					.getString(R.string.help_check_version));
			checkUpdateItem.put("show_detail_icon", false);
			mFunctionList.add(checkUpdateItem);

			// 关于
			Map<String, Object> infoItem = new HashMap<String, Object>();
			infoItem.put("icon", ((Activity) context).getResources()
					.getDrawable(R.drawable.info_icon));
			infoItem.put("title", ((Activity) context).getResources()
					.getString(R.string.help_about));
			infoItem.put("show_detail_icon", true);
			mFunctionList.add(infoItem);

			// 意见反馈
			Map<String, Object> feedbackItem = new HashMap<String, Object>();
			feedbackItem.put("icon", ((Activity) context).getResources()
					.getDrawable(R.drawable.feedback_icon));
			feedbackItem.put("title", ((Activity) context).getResources()
					.getString(R.string.more_feedback));
			feedbackItem.put("show_detail_icon", true);
			mFunctionList.add(feedbackItem);

			// 注销当前用户
			Map<String, Object> logoutItem = new HashMap<String, Object>();
			logoutItem.put("icon", ((Activity) context).getResources()
					.getDrawable(R.drawable.logout_icon));
			logoutItem.put("title", ((Activity) context).getResources()
					.getString(R.string.more_logout));
			logoutItem.put("show_detail_icon", false);
			mFunctionList.add(logoutItem);
		}
	}

	@Override
	public int getCount() {

		return mFunctionList == null ? 0 : mFunctionList.size();
	}

	@Override
	public Object getItem(int position) {
		return mFunctionList == null ? null : mFunctionList.get(position);
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		ListItemHolder itemHolder = null;
		if (convertView == null) {
			itemHolder = new ListItemHolder();
			convertView = mLayout.inflate(R.layout.function_item, null);

			// 初始化控件对象
			itemHolder.iconImage = (ImageView) convertView
					.findViewById(R.id.function_icon);
			itemHolder.titleText = (TextView) convertView
					.findViewById(R.id.function_title);
			itemHolder.detailImage = (ImageView) convertView
					.findViewById(R.id.function_select_icon);
			// 填充数据
			setHolderContent(itemHolder, position);

			// 设置控件集到convertView
			convertView.setTag(itemHolder);
		} else {// 复用convertView，只为其holder重新填充数据
			itemHolder = (ListItemHolder) convertView.getTag();
			setHolderContent(itemHolder, position);
		}
		return convertView;
	}

	private void setHolderContent(ListItemHolder holder, int position) {
		holder.iconImage.setImageBitmap(((BitmapDrawable) mFunctionList.get(
				position).get("icon")).getBitmap());
		holder.titleText.setText((String) mFunctionList.get(position).get(
				"title"));
		if (!(Boolean) mFunctionList.get(position).get("show_detail_icon")) {
			holder.detailImage.setVisibility(View.INVISIBLE);
		}
	}

}
