﻿package com.fynas.bookplan;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import org.json.JSONException;
import org.json.JSONObject;

import com.fynas.android.util.DateTime;
import com.fynas.android.util.HttpRequest;
import com.fynas.bookplan.common.AuthActivity;
import com.fynas.bookplan.common.Config;
import com.fynas.bookplan.common.User;
import com.fynas.android.model.JsonResult;
import com.fynas.bookplan.model.Plan;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;
import com.nostra13.universalimageloader.core.ImageLoader;
import android.app.DatePickerDialog;
import android.app.DatePickerDialog.OnDateSetListener;
import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.CompoundButton.OnCheckedChangeListener;

public class PlanDetailActivity extends AuthActivity {

	private Plan plan;
	private Button saveButton;

	@Override
	protected void onAuthSuccess(Bundle savedInstanceState) {

		setContentView(R.layout.activity_plan_detail);

		saveButton = (Button) findViewById(R.id.detail_save);
		saveButton.setVisibility(View.GONE);

		// 加载详细信息
		final int planId = (int) getIntent().getLongExtra("id", 0);
		HttpRequest.get(Config.Api.URL_GET_PLAN,
				new RequestParams("id", Integer.toString(planId)),
				new AsyncHttpResponseHandler() {
					public void onSuccess(String response) {

						User user = User.getInstance(getApplicationContext());
						Plan plan = new Plan();
						JsonResult result = new JsonResult(response);
						if (result.isOk()) {
							try {
								JSONObject detail = result.getJsonObject()
										.getJSONObject("data");
								plan.parseJson(detail);
								fill(plan);
							} catch (JSONException e) {
								Toast.makeText(PlanDetailActivity.this,
										R.string.error_load_data,
										Toast.LENGTH_SHORT).show();
								e.printStackTrace();
							}

						} else if (result.getStatus() == JsonResult.STATUS_NOT_LOGIN) {
							user.logout();
							startActivity(new Intent(PlanDetailActivity.this,
									LoginActivity.class));
							finish();
						}
					}
				});
	}

	private void fill(Plan plan) {
		this.plan = plan;
		TextView name = (TextView) findViewById(R.id.book_name);
		name.setText(plan.getBookName());

		TextView author = (TextView) findViewById(R.id.book_author);
		author.setText(plan.getBookAuthor());

		if (plan.getBookCoverUrl() != null) {
			ImageView coverImage = (ImageView) findViewById(R.id.detail_book_cover);
			ImageLoader imageLoader = ImageLoader.getInstance();
			imageLoader.displayImage(plan.getBookCoverUrl(), coverImage);
		}

		TextView startDateText = (TextView) findViewById(R.id.detail_start_date);
		EditText startDateEdit = (EditText) findViewById(R.id.detail_edit_start_date);
		EditText planEndDateEdit = (EditText) findViewById(R.id.detail_edit_plan_end_date);
		EditText endDateEdit = (EditText) findViewById(R.id.detail_edit_end_date);

		if (plan.getStatus() == Plan.STATUS_READING
				|| plan.getStatus() == Plan.STATUS_READ) {
			startDateText.setVisibility(View.VISIBLE);
			startDateEdit.setText(DateTime.dateToString(plan.getStartDate(),
					DateTime.FORMAT_STRING_DATE));
			startDateEdit.setVisibility(View.VISIBLE);
		} else {
			startDateText.setVisibility(View.GONE);
			startDateEdit.setVisibility(View.GONE);
		}

		planEndDateEdit.setText(DateTime.dateToString(plan.getPlanEndDate(),
				DateTime.FORMAT_STRING_DATE));
		planEndDateEdit.setVisibility(View.VISIBLE);

		TextView endDateText = (TextView) findViewById(R.id.detail_end_date);
		if (plan.getStatus() == Plan.STATUS_READ) {
			endDateText.setVisibility(View.VISIBLE);
			endDateEdit.setText(DateTime.dateToString(plan.getEndDate(),
					DateTime.FORMAT_STRING_DATE));
			endDateEdit.setVisibility(View.VISIBLE);
		} else {
			endDateText.setVisibility(View.GONE);
			endDateEdit.setVisibility(View.GONE);
		}

		// 隐藏loading图标
		ProgressBar loading = (ProgressBar) findViewById(R.id.load_plan_detail_progress);
		loading.setVisibility(View.GONE);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activity_plan_detail, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		if (item.getItemId() == R.id.menu_edit) {
			if (this.plan == null) {
				return false;
			}
			EditText planEndDateEdit = (EditText) findViewById(R.id.detail_edit_plan_end_date);
			CheckBox isMarkReadCheck = (CheckBox) findViewById(R.id.is_mark_read);
			CheckBox isMarkReadingCheck = (CheckBox) findViewById(R.id.is_mark_reading);

			// 启用计划读完时间编辑功能并挂载事件
			planEndDateEdit.setEnabled(true);
			planEndDateEdit.setOnClickListener(planEndDateClickListener);
			// 显示改变状态的复选框
			if (this.plan.getStatus() == Plan.STATUS_WANT) {
				isMarkReadingCheck.setVisibility(View.VISIBLE);
				isMarkReadingCheck
						.setOnCheckedChangeListener(isMarkReadingCheckedChangeListener);
			} else if (this.plan.getStatus() == Plan.STATUS_READING) {
				isMarkReadCheck
						.setOnCheckedChangeListener(isMarkReadCheckedChangeListener);
				isMarkReadCheck.setVisibility(View.VISIBLE);
			}

			// 挂载保存事件
			saveButton.setOnClickListener(saveClickListener);
			// 显示保存按钮
			saveButton.setVisibility(View.VISIBLE);
		}
		return true;

	}

	/**
	 * 点击保存按钮
	 */
	private OnClickListener saveClickListener = new OnClickListener() {

		@Override
		public void onClick(View view) {
			ProgressBar loading = (ProgressBar) findViewById(R.id.load_plan_detail_progress);
			loading.setVisibility(View.VISIBLE);
			EditText startDateEdit = (EditText) findViewById(R.id.detail_edit_start_date);
			EditText planEndDateEdit = (EditText) findViewById(R.id.detail_edit_plan_end_date);
			EditText endDateEdit = (EditText) findViewById(R.id.detail_edit_end_date);
			CheckBox isMarkReadCheck = (CheckBox) findViewById(R.id.is_mark_read);
			CheckBox isMarkReadingCheck = (CheckBox) findViewById(R.id.is_mark_reading);
			Map<String, String> params = new HashMap<String, String>();
			if (plan.getStatus() == Plan.STATUS_WANT) {// 当前是想读，可以变成想读或在读
				if (isMarkReadingCheck.isChecked()) {// 变成在读
					params.put("id", Integer.toString(plan.getId()));
					params.put("dest_status",
							Integer.toString(Plan.STATUS_READING));
					params.put("plan_end_date", planEndDateEdit.getText()
							.toString());
					params.put("start_date", startDateEdit.getText().toString());
				} else {
					params.put("id", Integer.toString(plan.getId()));
					params.put("dest_status",
							Integer.toString(Plan.STATUS_READING));
					params.put("plan_end_date", planEndDateEdit.getText()
							.toString());

				}
			} else if (plan.getStatus() == Plan.STATUS_READING) {// 当前是在读，可以变成在读或已读
				if (isMarkReadCheck.isChecked()) {// 变成已读
					if (DateTime.isLaterThan(
							startDateEdit.getText().toString(), endDateEdit
									.getText().toString())) {
						loading.setVisibility(View.GONE);
						Toast.makeText(PlanDetailActivity.this, "结束日期不能早于开始日期",
								Toast.LENGTH_SHORT).show();
						return;
					}

					params.put("id", Integer.toString(plan.getId()));
					params.put("dest_status",
							Integer.toString(Plan.STATUS_READING));
					params.put("plan_end_date", planEndDateEdit.getText()
							.toString());
					params.put("end_date", endDateEdit.getText().toString());

				} else {
					// 异步执行保存
					params.put("id", Integer.toString(plan.getId()));
					params.put("dest_status",
							Integer.toString(Plan.STATUS_READING));
					params.put("plan_end_date", planEndDateEdit.getText()
							.toString());
					params.put("start_date", startDateEdit.getText().toString());
				}
			}

			if (params.containsKey("id")) {
				HttpRequest.post(Config.Api.URL_UPDATE_PLAN, new RequestParams(
						params), new AsyncHttpResponseHandler() {
					public void onFinish() {
						ProgressBar loading = (ProgressBar) findViewById(R.id.load_plan_detail_progress);
						loading.setVisibility(View.GONE);
					}

					public void onSuccess(String response) {
						JsonResult result = new JsonResult(response);
						Toast.makeText(PlanDetailActivity.this,
								result.isOk() ? "保存成功" : "保存失败",
								Toast.LENGTH_SHORT).show();
					};
				});
			}

		}
	};

	/**
	 * 选中标记为已读
	 */
	private OnCheckedChangeListener isMarkReadCheckedChangeListener = new OnCheckedChangeListener() {

		@Override
		public void onCheckedChanged(CompoundButton buttonView,
				boolean isChecked) {
			EditText endDateEdit = (EditText) findViewById(R.id.detail_edit_end_date);
			TextView endDateText = (TextView) findViewById(R.id.detail_end_date);
			endDateText.setVisibility(isChecked ? View.VISIBLE : View.GONE);
			endDateEdit.setText(DateTime.todayString());
			endDateEdit.setVisibility(isChecked ? View.VISIBLE : View.GONE);
			endDateEdit.setEnabled(isChecked);
			endDateEdit.setOnClickListener(isChecked ? endDateClickListener
					: null);
		}

	};

	/**
	 * 点击编辑结束时间
	 */
	private OnClickListener endDateClickListener = new OnClickListener() {

		@Override
		public void onClick(View view) {
			Calendar calendar = DateTime.stringToCalendar(((EditText) view)
					.getText().toString());
			new DatePickerDialog(PlanDetailActivity.this, endDateSetListener,
					calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH),
					calendar.get(Calendar.DAY_OF_MONTH)).show();

		}

	};

	private OnDateSetListener endDateSetListener = new OnDateSetListener() {

		@Override
		public void onDateSet(DatePicker view, int year, int monthOfYear,
				int dayOfMonth) {
			EditText endDateEdit = (EditText) findViewById(R.id.detail_edit_end_date);
			endDateEdit.setText(DateTime
					.toString(year, monthOfYear, dayOfMonth));

		}

	};

	/**
	 * 选中标记为开始读
	 */
	private OnCheckedChangeListener isMarkReadingCheckedChangeListener = new OnCheckedChangeListener() {

		@Override
		public void onCheckedChanged(CompoundButton buttonView,
				boolean isChecked) {
			EditText startDateEdit = (EditText) findViewById(R.id.detail_edit_start_date);
			TextView startDateText = (TextView) findViewById(R.id.detail_start_date);
			startDateText.setVisibility(isChecked ? View.VISIBLE : View.GONE);
			startDateEdit.setText(DateTime.todayString());
			startDateEdit.setVisibility(isChecked ? View.VISIBLE : View.GONE);
			startDateEdit.setEnabled(isChecked);
			startDateEdit.setOnClickListener(isChecked ? startDateClickListener
					: null);
		}

	};
	/**
	 * 点击编辑开始时间
	 */
	private OnClickListener startDateClickListener = new OnClickListener() {

		@Override
		public void onClick(View view) {
			Calendar calendar = DateTime.stringToCalendar(((EditText) view)
					.getText().toString());
			new DatePickerDialog(PlanDetailActivity.this, startDateSetListener,
					calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH),
					calendar.get(Calendar.DAY_OF_MONTH)).show();

		}

	};

	/**
	 * 开始时间被设置
	 */
	private OnDateSetListener startDateSetListener = new OnDateSetListener() {

		@Override
		public void onDateSet(DatePicker view, int year, int monthOfYear,
				int dayOfMonth) {
			EditText startDateEdit = (EditText) findViewById(R.id.detail_edit_start_date);
			startDateEdit.setText(DateTime.toString(year, monthOfYear,
					dayOfMonth));

		}

	};

	/**
	 * 点击计划结束日期
	 */
	private OnClickListener planEndDateClickListener = new OnClickListener() {

		@Override
		public void onClick(View view) {
			Calendar calendar = DateTime.stringToCalendar(((EditText) view)
					.getText().toString());
			new DatePickerDialog(PlanDetailActivity.this,
					planEndDateSetListener, calendar.get(Calendar.YEAR),
					calendar.get(Calendar.MONTH),
					calendar.get(Calendar.DAY_OF_MONTH)).show();

		}

	};

	/**
	 * 计划结束日期被设置
	 */
	private OnDateSetListener planEndDateSetListener = new OnDateSetListener() {

		@Override
		public void onDateSet(DatePicker view, int year, int monthOfYear,
				int dayOfMonth) {
			EditText planEndDateEdit = (EditText) findViewById(R.id.detail_edit_plan_end_date);
			planEndDateEdit.setText(DateTime.toString(year, monthOfYear,
					dayOfMonth));

		}
	};

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			finish();
			return true;
		} else {
			return super.onKeyDown(keyCode, event);
		}
	}

}
