﻿package com.fynas.bookplan;

import java.util.List;
import com.fynas.bookplan.model.Plan;
import com.nostra13.universalimageloader.core.ImageLoader;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

public class PlanListAdapter extends BaseAdapter {

	private List<Plan> mPlanList; // 图书信息集合
	private LayoutInflater mLayout; // 视图容器

	public final class ListItemHolder { // 自定义控件集合
		public ImageView coverImage;
		public TextView nameText;
		public TextView authorText;
	}

	public PlanListAdapter(Context context, List<Plan> list) {
		mLayout = LayoutInflater.from(context);
		mPlanList = list;
	}

	public List<Plan> getPlanList() {
		return mPlanList;
	}

	@Override
	public int getCount() {
		return mPlanList == null ? 0 : mPlanList.size();
	}

	@Override
	public Plan getItem(int position) {
		return mPlanList.get(position);
	}

	@Override
	public long getItemId(int position) {
		return getItem(position).getId();
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		ListItemHolder itemHolder = null;
		if (convertView == null) {// 初始化convertView
			itemHolder = new ListItemHolder();
			// 获取list_item布局文件的视图
			convertView = mLayout.inflate(R.layout.book_item, null);
			// 初始化控件对象
			itemHolder.coverImage = (ImageView) convertView
					.findViewById(R.id.book_cover);
			itemHolder.nameText = (TextView) convertView
					.findViewById(R.id.book_name);
			itemHolder.authorText = (TextView) convertView
					.findViewById(R.id.book_author);

			setHolderContent(itemHolder, position);

			// 设置控件集到convertView
			convertView.setTag(itemHolder);
		} else {// 复用convertView，只为其holder重新填充数据
			itemHolder = (ListItemHolder) convertView.getTag();
			setHolderContent(itemHolder, position);
		}
		return convertView;
	}

	/**
	 * 向item填充数据内容
	 * 
	 * @param holder
	 *            item容器
	 * @param position
	 *            item的位置
	 */
	private void setHolderContent(ListItemHolder holder, int position) {
		if (mPlanList.get(position).getBookCoverUrl() != null) {
			ImageLoader imageLoader = ImageLoader.getInstance();
			imageLoader.displayImage(mPlanList.get(position).getBookCoverUrl(),
					holder.coverImage);
		}
		holder.nameText.setText((String) mPlanList.get(position).getBookName());
		holder.authorText.setText((String) mPlanList.get(position)
				.getBookAuthor());
	}

	@Override
	public void notifyDataSetChanged() {
		super.notifyDataSetChanged();
	}
}
