package com.fynas.bookplan.model;

import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.json.JSONException;
import org.json.JSONObject;

import com.fynas.android.model.IModel;
import com.fynas.android.util.DateTime;
import com.fynas.bookplan.common.Config;

public class Plan implements IModel {

	private int id = 0;
	private int bookId;
	private String bookName;
	private String bookAuthor;
	private String bookCoverName;
	private Date startDate;
	private Date planEndDate;
	private Date endDate;

	public final static int STATUS_READING = 1;
	public final static int STATUS_WANT = 0;
	public final static int STATUS_READ = 2;

	public String getBookCoverName() {
		return bookCoverName;
	}

	public String getBookCoverUrl() {
		return getBookCoverName() == null ? null : String.format(
				Config.Api.URL_COVER, bookCoverName);
	}

	public void setBookCoverName(String name) {
		bookCoverName = name.length() == 32 ? name : null;
	}

	public int getStatus() {
		if (endDate != null) {
			return Plan.STATUS_READ;
		} else if (startDate != null && planEndDate != null && endDate == null) {
			return Plan.STATUS_READING;
		} else if (startDate == null && planEndDate != null && endDate == null) {
			return Plan.STATUS_WANT;
		} else {
			return -1;
		}

	}

	public Date getStartDate() {
		return startDate;
	}

	public void setStartDate(Date date) {
		this.startDate = date;
	}

	public void setStartDate(String date) {
		Pattern pattern = Pattern.compile("\\d{4}\\-\\d{2}\\-\\d{2}");
		Matcher matcher = pattern.matcher(date);
		this.startDate = matcher.find() ? DateTime.stringToDate(date,
				DateTime.FORMAT_STRING_DATE) : null;
	}

	public Date getPlanEndDate() {
		return planEndDate;
	}

	public void setPlanEndDate(Date date) {
		this.planEndDate = date;
	}

	public void setPlanEndDate(String date) {
		Pattern pattern = Pattern.compile("\\d{4}\\-\\d{2}\\-\\d{2}");
		Matcher matcher = pattern.matcher(date);
		this.planEndDate = matcher.find() ? DateTime.stringToDate(date,
				DateTime.FORMAT_STRING_DATE) : null;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setEndDate(Date date) {
		this.endDate = date;
	}

	public void setEndDate(String date) {
		Pattern pattern = Pattern.compile("\\d{4}\\-\\d{2}\\-\\d{2}");
		Matcher matcher = pattern.matcher(date);
		this.endDate = matcher.find() ? DateTime.stringToDate(date,
				DateTime.FORMAT_STRING_DATE) : null;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public int getBookId() {
		return bookId;
	}

	public void setBookId(int id) {
		this.bookId = id;
	}

	public String getBookName() {
		return bookName;
	}

	public void setBookName(String name) {
		this.bookName = name;
	}

	public String getBookAuthor() {
		return bookAuthor;
	}

	public void setBookAuthor(String author) {
		this.bookAuthor = author;
	}

	@Override
	public void parseJson(JSONObject json) throws JSONException {
		setId(json.getInt("id"));
		setBookName(json.getString("book_name"));
		setBookAuthor(json.getString("book_author"));
		setBookCoverName(json.getString("book_cover"));
		setBookId(Integer.parseInt(json.getString("book_id")));
		if (json.has("start_date")) {
			setStartDate(json.getString("start_date"));
		}
		if (json.has("plan_end_date")) {
			setPlanEndDate(json.getString("plan_end_date"));
		}
		if (json.has("end_date")) {
			setEndDate(json.getString("end_date"));
		}
	}

}
