﻿package com.fynas.bookplan.view;

import com.fynas.bookplan.R;
import android.content.Context;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.LinearInterpolator;
import android.view.animation.RotateAnimation;
import android.widget.AbsListView;
import android.widget.ImageView;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.AbsListView.OnScrollListener;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class PullToRefreshListView extends ListView implements OnScrollListener {

	private static final int REFRESH_STATE_TAP_TO_REFRESH = 1;// 点击刷新
	private static final int REFRESH_STATE_PULL_TO_REFRESH = 2;// 下拉刷新
	private static final int REFRESH_STATE_RELEASE_TO_REFRESH = 3;// 松开刷新
	private static final int REFRESH_STATE_REFRESHING = 4;// 正在刷新

	private OnRefreshListener mOnRefreshListener;

	/**
	 * 收到通知的监听器每次滚动列表。
	 */
	private OnScrollListener mOnScrollListener;
	private LayoutInflater mInflater;

	private RelativeLayout mRefreshLayout;
	private TextView mRefreshText;
	private ImageView mRefreshImage;
	private ProgressBar mRefreshProgress;
	private TextView mRefreshLastUpdateText;

	/**
	 * 当前滚动状态
	 */
	private int mCurrentScrollState;

	/**
	 * 当前刷新状态
	 */
	private int mRefreshState;

	private RotateAnimation mFlipAnimation;
	private RotateAnimation mReverseFlipAnimation;

	/**
	 * 头部视图高度
	 */
	private int mRefreshViewHeight;
	private int mRefreshOriginalTopPadding;
	private int mLastMotionY;

	/**
	 * 是否反弹
	 */
	private boolean mBounceHack;

	public PullToRefreshListView(Context context) {
		super(context);
		init(context);
	}

	public PullToRefreshListView(Context context, AttributeSet attrs) {
		super(context, attrs);
		init(context);
	}

	public PullToRefreshListView(Context context, AttributeSet attrs,
			int defStyle) {
		super(context, attrs, defStyle);
		init(context);
	}

	/**
	 * 初始化listview
	 * 
	 * @param context
	 *            上下文
	 */
	private void init(Context context) {
		// 创建旋转动画
		mFlipAnimation = new RotateAnimation(0, -180,
				RotateAnimation.RELATIVE_TO_SELF, 0.5f,
				RotateAnimation.RELATIVE_TO_SELF, 0.5f);
		mFlipAnimation.setInterpolator(new LinearInterpolator());
		mFlipAnimation.setDuration(250);
		mFlipAnimation.setFillAfter(true);
		// 创建反转动画
		mReverseFlipAnimation = new RotateAnimation(-180, 0,
				RotateAnimation.RELATIVE_TO_SELF, 0.5f,
				RotateAnimation.RELATIVE_TO_SELF, 0.5f);
		mReverseFlipAnimation.setInterpolator(new LinearInterpolator());
		mReverseFlipAnimation.setDuration(250);
		mReverseFlipAnimation.setFillAfter(true);

		mInflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		// inflate头部item布局
		mRefreshLayout = (RelativeLayout) mInflater.inflate(
				R.layout.pull_to_refresh_header, this, false);
		mRefreshLayout.setOnClickListener(new OnClickRefreshListener());

		// 初始化刷新图标控件
		mRefreshImage = (ImageView) mRefreshLayout
				.findViewById(R.id.pull_to_refresh_image);
		mRefreshImage.setMinimumHeight(50);
		// 初始化文本控件
		mRefreshText = (TextView) mRefreshLayout
				.findViewById(R.id.pull_to_refresh_text);
		mRefreshLastUpdateText = (TextView) mRefreshLayout
				.findViewById(R.id.pull_to_refresh_updated_at);

		// 初始化loading图标
		mRefreshProgress = (ProgressBar) mRefreshLayout
				.findViewById(R.id.pull_to_refresh_progress);

		mRefreshOriginalTopPadding = mRefreshLayout.getPaddingTop();

		// 初始化状态
		mRefreshState = REFRESH_STATE_TAP_TO_REFRESH;

		// 将头部添加到list view中
		addHeaderView(mRefreshLayout);

		super.setOnScrollListener(this);

		measureView(mRefreshLayout);
		mRefreshViewHeight = mRefreshLayout.getMeasuredHeight();
	}

	@Override
	protected void onAttachedToWindow() {
		super.onAttachedToWindow();
		setSelection(1);
	}

	@Override
	public void setAdapter(ListAdapter adapter) {
		super.setAdapter(adapter);
		setSelection(1);
	}

	/**
	 * 设置将接收通知的侦听器，每次滚动列表。
	 * 
	 * @param onScrollListener
	 */
	@Override
	public void setOnScrollListener(
			AbsListView.OnScrollListener onScrollListener) {
		mOnScrollListener = onScrollListener;
	}

	/**
	 * 注册一个回调函数被调用时，此列表应被刷新。
	 * 
	 * @param onRefreshListener
	 *            The callback to run.
	 */
	public void setOnRefreshListener(OnRefreshListener onRefreshListener) {
		mOnRefreshListener = onRefreshListener;
	}

	/**
	 * 设置显示最后成功更新的时间
	 * 
	 * @param lastUpdatedTime
	 */
	public void setLastUpdated(CharSequence lastUpdatedTime) {
		if (lastUpdatedTime != null) {
			mRefreshLastUpdateText.setText(getResources().getString(
					R.string.pull_to_refresh_update_at)
					+ lastUpdatedTime);
			mRefreshLastUpdateText.setVisibility(View.VISIBLE);
		} else {
			mRefreshLastUpdateText.setVisibility(View.GONE);
		}
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		final int y = (int) event.getY();
		mBounceHack = false;

		switch (event.getAction()) {
		case MotionEvent.ACTION_UP:
			if (!isVerticalScrollBarEnabled()) {
				setVerticalScrollBarEnabled(true);
			}
			if (getFirstVisiblePosition() == 0
					&& mRefreshState != REFRESH_STATE_REFRESHING) {
				if ((mRefreshLayout.getBottom() >= mRefreshViewHeight || mRefreshLayout
						.getTop() >= 0)
						&& mRefreshState == REFRESH_STATE_RELEASE_TO_REFRESH) {
					// Initiate the refresh
					mRefreshState = REFRESH_STATE_REFRESHING;
					refresh();
				} else if (mRefreshLayout.getBottom() < mRefreshViewHeight
						|| mRefreshLayout.getTop() < 0) {
					// Abort refresh and scroll down below the refresh view
					resetHeader();
					setSelection(1);
				}
			}
			break;
		case MotionEvent.ACTION_DOWN:
			mLastMotionY = y;
			break;
		case MotionEvent.ACTION_MOVE:
			setHeaderPadding(event);
			break;
		}
		return super.onTouchEvent(event);
	}

	private void setHeaderPadding(MotionEvent event) {

		if (mRefreshState != REFRESH_STATE_RELEASE_TO_REFRESH) {
			return;
		}

		int pointerCount = event.getHistorySize();
		if (pointerCount > 0 && isVerticalFadingEdgeEnabled()) {
			setVerticalScrollBarEnabled(false);
		}

		for (int p = 0; p < pointerCount; p++) {
			int topPadding = (((int) event.getHistoricalY(p) - mLastMotionY) - mRefreshViewHeight);
			if (topPadding <= mRefreshOriginalTopPadding) {// 当顶边距小于原始顶边距时不重设置顶边距，以防止边距跳动带来的刷新状态反复
				continue;
			}
			//将顶边距增量除以一个数值，增加继续下拉更紧的感觉，更逼真的算法可以将增量开平方等
			topPadding = mRefreshOriginalTopPadding
					+ (int) ((topPadding - mRefreshOriginalTopPadding) / 1.7);
			mRefreshLayout
					.setPadding(
							mRefreshLayout.getPaddingLeft(),
							topPadding > mRefreshOriginalTopPadding * 3 ? mRefreshOriginalTopPadding * 3
									: topPadding, mRefreshLayout
									.getPaddingRight(), mRefreshLayout
									.getPaddingBottom());
		}
	}

	/**
	 * 重设头部，恢复到点击刷新状态
	 */
	private void resetHeader() {
		if (mRefreshState != REFRESH_STATE_TAP_TO_REFRESH) {
			mRefreshState = REFRESH_STATE_TAP_TO_REFRESH;

			mRefreshLayout.setPadding(mRefreshLayout.getPaddingLeft(),
					mRefreshOriginalTopPadding,
					mRefreshLayout.getPaddingRight(),
					mRefreshLayout.getPaddingBottom());

			// 刷新视图文本拉标签
			mRefreshText.setText(R.string.pull_to_refresh_tap);
			mRefreshImage.setImageResource(R.drawable.ico_arrow_down);
			// 清除旋转动画
			mRefreshImage.clearAnimation();
			// 隐藏进度条和箭头
			mRefreshImage.setVisibility(View.GONE);
			mRefreshProgress.setVisibility(View.GONE);
		}
	}

	private void measureView(View child) {
		ViewGroup.LayoutParams params = child.getLayoutParams();
		if (params == null) {
			params = new ViewGroup.LayoutParams(
					ViewGroup.LayoutParams.MATCH_PARENT,
					ViewGroup.LayoutParams.WRAP_CONTENT);
		}

		int childWidthSpec = ViewGroup.getChildMeasureSpec(0, 0 + 0,
				params.width);
		int childHeightSpec;
		if (params.height > 0) {
			childHeightSpec = MeasureSpec.makeMeasureSpec(params.height,
					MeasureSpec.EXACTLY);
		} else {
			childHeightSpec = MeasureSpec.makeMeasureSpec(0,
					MeasureSpec.UNSPECIFIED);
		}
		child.measure(childWidthSpec, childHeightSpec);
	}

	@Override
	public void onScroll(AbsListView view, int firstVisibleItem,
			int visibleItemCount, int totalItemCount) {
		// 当刷新view完全可见时，改变提示文字，翻转箭头。
		if (mCurrentScrollState == SCROLL_STATE_TOUCH_SCROLL
				&& mRefreshState != REFRESH_STATE_REFRESHING) {
			if (firstVisibleItem == 0) {// 第一个可视项为头部时
				// 显示箭头
				mRefreshImage.setVisibility(View.VISIBLE);

				if ((mRefreshLayout.getBottom() >= mRefreshViewHeight || mRefreshLayout
						.getTop() >= 0)
						&& mRefreshState != REFRESH_STATE_RELEASE_TO_REFRESH) {// 当头部被完全拉出时
					mRefreshText.setText(R.string.pull_to_refresh_release);
					mRefreshImage.clearAnimation();
					mRefreshImage.startAnimation(mFlipAnimation);
					// 设置当前状态为松开刷新
					mRefreshState = REFRESH_STATE_RELEASE_TO_REFRESH;

				} else if (mRefreshLayout.getBottom() < mRefreshViewHeight
						&& mRefreshState != REFRESH_STATE_PULL_TO_REFRESH) {// 当头部被拉出，但没有完全拉出时
					mRefreshText.setText(R.string.pull_to_refresh);
					if (mRefreshState != REFRESH_STATE_TAP_TO_REFRESH) {
						mRefreshImage.clearAnimation();
						mRefreshImage.startAnimation(mReverseFlipAnimation);
					}
					// 设置当前状态为下拉刷新
					mRefreshState = REFRESH_STATE_PULL_TO_REFRESH;

				}
			} else {// 第一个可见项不是头部
				resetHeader();
			}
		} else if (mCurrentScrollState == SCROLL_STATE_FLING
				&& firstVisibleItem == 0
				&& mRefreshState != REFRESH_STATE_REFRESHING) {
			setSelection(1);
			mBounceHack = true;
		} else if (mBounceHack && mCurrentScrollState == SCROLL_STATE_FLING) {
			setSelection(1);
		}

		if (mOnScrollListener != null) {
			mOnScrollListener.onScroll(view, firstVisibleItem,
					visibleItemCount, totalItemCount);
		}
	}

	@Override
	public void onScrollStateChanged(AbsListView view, int scrollState) {
		mCurrentScrollState = scrollState;

		if (mCurrentScrollState == SCROLL_STATE_IDLE) {
			mBounceHack = false;
		}

		if (mOnScrollListener != null) {
			mOnScrollListener.onScrollStateChanged(view, scrollState);
		}
	}

	public void onPreRefresh() {

		mRefreshLayout.setPadding(mRefreshLayout.getPaddingLeft(),
				mRefreshOriginalTopPadding, mRefreshLayout.getPaddingRight(),
				mRefreshLayout.getPaddingBottom());

		mRefreshImage.setVisibility(View.GONE);
		// 清空图像
		mRefreshImage.setImageDrawable(null);
		mRefreshProgress.setVisibility(View.VISIBLE);
		mRefreshText.setText(R.string.pull_to_refresh_refreshing);

		mRefreshState = REFRESH_STATE_REFRESHING;
	}

	public void refresh() {
		onPreRefresh();
		if (mOnRefreshListener != null) {
			mOnRefreshListener.onRefresh();
		}
	}

	/**
	 * 重置列表刷新后到正常状态
	 * 
	 * @param lastUpdated
	 */
	public void onRefreshComplete(CharSequence lastUpdated) {
		setLastUpdated(lastUpdated);
		onRefreshComplete();
	}

	/**
	 * 重置列表刷新后到正常状态。
	 */
	public void onRefreshComplete() {
		resetHeader();
		mRefreshProgress.setVisibility(View.GONE);
		// 如果刷新视图可见
		if (mRefreshLayout.getBottom() > 0) {
			invalidateViews();
			mRefreshText.setText(R.string.pull_to_refresh_tap);
			setSelection(1);
		}
	}

	/**
	 * 点击刷新,这主要是用于只有条数据的列表中
	 */
	private class OnClickRefreshListener implements OnClickListener {

		@Override
		public void onClick(View v) {
			if (mRefreshState != REFRESH_STATE_REFRESHING) {
				refresh();
			}
		}

	}

	/**
	 * 刷新侦听器接口
	 * 
	 * @author SuperApple
	 * 
	 */
	public interface OnRefreshListener {
		/**
		 * 列表被刷新时调用 A call to {@link PullToRefreshListView #onRefreshComplete()}
		 * is expected to indicate that the refresh has completed.
		 */
		public void onRefresh();
	}
}
